---------------------------------------------------------------------------
--                                                                       --
--  ast2dot is an ASIS application that outputs the AST in dot format.   --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--        Copyright (c) 2006,2009 Raul Fechete, Georg Kienesberger       --
--           Copyright (c) 2000, Free Software Foundation, Inc.          --
--                                                                       --
---------------------------------------------------------------------------
-- This file is part of ast2dot, and is based on the ASIS Application    --
-- Templates. Date of change: 20 December 2006                           --
--                                                                       --
-- ast2dot is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 2 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- ast2dot is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with ast2dot; if not, write to the Free Software                --
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 --
-- USA                                                                   --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Wide_Text_IO;
with Ada.Characters.Handling;
with Ada.Exceptions;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;

with Asis.Compilation_Units;
with Asis.Exceptions;
with Asis.Errors;
with Asis.Implementation;

with Unit_Processing;

with Transformation_Control;

with Process_Output;

package body Context_Processing is

   -----------------------------
   -- Get_Unit_From_File_Name --
   -----------------------------

   function Get_Unit_From_File_Name
     (Ada_File_Name : String;
      The_Context   : Asis.Context)
      return Asis.Compilation_Unit
   is
   begin

      return Asis.Nil_Compilation_Unit;

   end Get_Unit_From_File_Name;

   ---------------------
   -- Process_Context --
   ---------------------

   procedure Process_Context
     (The_Context : Asis.Context;
      Trace       : Boolean := False)
   is
      Units : Asis.Compilation_Unit_List :=
         Asis.Compilation_Units.Compilation_Units (The_Context);

      Next_Unit        : Asis.Compilation_Unit := Asis.Nil_Compilation_Unit;
      Next_Unit_Origin : Asis.Unit_Origins     := Asis.Not_An_Origin;
      Next_Unit_Class  : Asis.Unit_Classes     := Asis.Not_A_Class;

      DigraphName: Unbounded_String;

   begin

      for J in Units'Range loop
         Next_Unit        := Units (J);
         Next_Unit_Class  := Asis.Compilation_Units.Unit_Class (Next_Unit);
         Next_Unit_Origin := Asis.Compilation_Units.Unit_Origin (Next_Unit);

         case Next_Unit_Origin is
            when Asis.An_Application_Unit =>

               Transformation_Control.ZeroId;

               DigraphName := To_Unbounded_String(Ada.Characters.Handling.To_String(Asis.Compilation_Units.Unit_Full_Name (Next_Unit)));

               case Next_Unit_Class is
                  when Asis.A_Public_Declaration |
                    Asis.A_Private_Declaration =>

                     DigraphName := DigraphName & " (spec)";

                  when Asis.A_Separate_Body =>
                     DigraphName := DigraphName & " (subunit)";

                  when Asis.A_Public_Body |
                    Asis.A_Public_Declaration_And_Body |
                    Asis.A_Private_Body =>

                     DigraphName := DigraphName & " (body)";

                  when others =>
                     DigraphName := DigraphName & " (???)";

               end case;

               Process_Output.BeginDigraph(To_String(DigraphName));

               Unit_Processing.Process_Unit (Next_Unit);

               Process_Output.EndDigraph;


            when others =>
               null;
         end case;

      end loop;

   exception

      when Ex : Asis.Exceptions.ASIS_Inappropriate_Context          |
                Asis.Exceptions.ASIS_Inappropriate_Container        |
                Asis.Exceptions.ASIS_Inappropriate_Compilation_Unit |
                Asis.Exceptions.ASIS_Inappropriate_Element          |
                Asis.Exceptions.ASIS_Inappropriate_Line             |
                Asis.Exceptions.ASIS_Inappropriate_Line_Number      |
                Asis.Exceptions.ASIS_Failed                         =>

         Ada.Wide_Text_IO.Put ("Process_Context : ASIS exception (");

         Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
                 Ada.Exceptions.Exception_Name (Ex)));

         Ada.Wide_Text_IO.Put (") is raised when processing unit ");

         Ada.Wide_Text_IO.Put
            (Asis.Compilation_Units.Unit_Full_Name (Next_Unit));

         Ada.Wide_Text_IO.New_Line;

         Ada.Wide_Text_IO.Put ("ASIS Error Status is ");

         Ada.Wide_Text_IO.Put
           (Asis.Errors.Error_Kinds'Wide_Image (Asis.Implementation.Status));

         Ada.Wide_Text_IO.New_Line;

         Ada.Wide_Text_IO.Put ("ASIS Diagnosis is ");
         Ada.Wide_Text_IO.New_Line;
         Ada.Wide_Text_IO.Put (Asis.Implementation.Diagnosis);
         Ada.Wide_Text_IO.New_Line;

         Asis.Implementation.Set_Status;

      when Ex : others =>

         Ada.Wide_Text_IO.Put ("Process_Context : ");

         Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
                 Ada.Exceptions.Exception_Name (Ex)));

         Ada.Wide_Text_IO.Put (" is raised (");

         Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
                 Ada.Exceptions.Exception_Information (Ex)));

         Ada.Wide_Text_IO.Put (")");
         Ada.Wide_Text_IO.New_Line;

         Ada.Wide_Text_IO.Put ("when processing unit");

         Ada.Wide_Text_IO.Put
            (Asis.Compilation_Units.Unit_Full_Name (Next_Unit));

         Ada.Wide_Text_IO.New_Line;

   end Process_Context;

end Context_Processing;
